import numpy as np 
from ur_analytic_ik import ur3, ur3e, ur5, ur5e, ur7e, ur8long, ur10, ur10e, ur12e, ur15, ur16e, ur18, ur20, ur30

import pytest 


UR_MODULES = [
    ur3, ur3e, ur5, ur5e, ur7e, ur8long,
    ur10, ur10e, ur12e, ur15, ur16e,
    ur18, ur20, ur30,
]

@pytest.mark.parametrize("ur", UR_MODULES)
def test_non_contiguous(ur):
    """
    test if the IK functions work for non-contiguous arrays. 
    This is not trivial as Eigen expects Row-major arrays, and numpy can create non-contiguous views which causes issues when passing them to Eigen.
    """
    rx = np.array([-1,0,0,0])
    ry = np.array([0,1,0,0])
    rz = np.array([0,0,-1,0])
    t = np.array([0.2,0.2,0.1,1])

    pose = np.array([rx, ry, rz, t]).T
    joint_solutions = ur.inverse_kinematics(pose)

    # Normalize the module name (handle extension module names like 'ur_analytic_ik_ext.ur3')
    module_short = ur.__name__.split('.')[-1]
    if module_short in ('ur3', 'ur3e', 'ur5', 'ur5e', 'ur7e'):
        assert len(joint_solutions) == 8, f"Expected 8 solutions for {ur.__name__}, got {len(joint_solutions)}"
    else:
        assert len(joint_solutions) >= 4, f"Expected at least 4 solutions for {ur.__name__}, got {len(joint_solutions)}"


    for joints in joint_solutions:
        eef_pose = np.array(ur.forward_kinematics(*joints.squeeze()))
        assert np.allclose(eef_pose, pose), f"Expected {pose}, got {eef_pose} for one of the joint configurations"



"""
use a set of (joint_config, pose) pairs  collected from a real robot to test if the IK/FK solutions correspond to the real robot.
"""

import numpy as np
import pytest
from ur_analytic_ik import ur5e

def read_poses(file):
    with  open(file, "r") as f:
        lines = f.readlines()
    joint_config_pose_pairs = []
    for line in lines:
        if len(line) == 1:
            continue
        joint_config, pose = line.split(";")[0], line.split(";")[1]
        joint_config = joint_config.split(",")
        # strip the brackets
        joint_config[0] = joint_config[0][1:]
        joint_config[-1] = joint_config[-1][:-1]
        joint_config = [float(j) for j in joint_config]
        
        pose = pose.split(",")
        # remove all brackets
        for i in range(len(pose)):
            pose[i] = pose[i].replace("[", "").replace("]", "")
        pose = [float(p) for p in pose]
        pose = np.array(pose).reshape(4, 4)
        joint_config_pose_pairs.append((joint_config, pose))

    return joint_config_pose_pairs

def test_solutions_match_real_ur5e():
    import pathlib
    file_path = pathlib.Path(__file__).parent / "data/ur5e_poses.txt"
    joint_config_pose_pairs = read_poses(str(file_path))
    for joint_config, pose in joint_config_pose_pairs:
      
        joint_solution = ur5e.inverse_kinematics_closest(pose, *joint_config)[0]
        assert np.isclose(joint_solution, joint_config,atol=1e-2).all()


        fk_pose = ur5e.forward_kinematics(*joint_config)
        assert np.linalg.norm(fk_pose[:3,3] - pose[:3,3]) < 3e-3 # 3mm tolerance
        


import sys, re, os

try:
    from skbuild import setup
    import nanobind
except ImportError:
    print(
        "The preferred way to invoke 'setup.py' is via pip, as in 'pip "
        "install .'. If you wish to run the setup script directly, you must "
        "first install the build dependencies listed in pyproject.toml!",
        file=sys.stderr,
    )
    raise

with open("README.md", "r") as fh:
    long_description = fh.read()


setup(
    packages=["ur_analytic_ik"],
    cmake_install_dir="ur_analytic_ik",
    include_package_data=True,
    long_description=long_description,
    long_description_content_type="text/markdown",
)
